@@ -1,7 +1,7 @@
 ﻿// drumkv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1211,7 +1211,7 @@ private:
 drumkv1_impl::drumkv1_impl (
 	drumkv1 *pDrumk, uint16_t nchannels, float srate )
 	: m_pDrumk(pDrumk),	m_controls(pDrumk), m_programs(pDrumk),
-		m_midi_in(pDrumk), m_bpm(180.0f), m_running(false)
+		m_midi_in(pDrumk), m_bpm(180.0f), m_nvoices(0), m_running(false)
 {
 	// allocate voice pool.
 	m_voices = new drumkv1_voice * [MAX_VOICES];
@@ -2310,8 +2310,8 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				// filters
 
 				if (dcf1_enabled) {
-					const float env1 = 0.5f * (1.0f + vel1
-						* *elem->dcf1.envelope * pv->dcf1_env.tick());
+					const float env1 = 0.5f
+						* (1.0f + *elem->dcf1.envelope * pv->dcf1_env.tick());
 					const float cutoff1 = drumkv1_sigmoid_1(*elem->dcf1.cutoff
 						* env1 * (1.0f + *elem->lfo1.cutoff * lfo1));
 					const float reso1 = drumkv1_sigmoid_1(*elem->dcf1.reso
@@ -2614,11 +2614,11 @@ void drumkv1::clearElements (void)
 }
 
 
-void drumkv1::setSampleFile ( const char *pszSampleFile )
+void drumkv1::setSampleFile ( const char *pszSampleFile, bool bSync )
 {
 	m_pImpl->setSampleFile(pszSampleFile);
 
-	updateSample();
+	if (bSync) updateSample();
 }
 
 const char *drumkv1::sampleFile (void) const
@@ -2652,7 +2652,7 @@ void drumkv1::setOffset ( bool bOffset, bool bSync )
 	m_pImpl->setOffset(bOffset);
 	m_pImpl->sampleOffsetSync();
 
-	if (bSync) updateSample();
+	if (bSync) updateOffsetRange();
 }
 
 bool drumkv1::isOffset (void) const
@@ -2661,13 +2661,13 @@ bool drumkv1::isOffset (void) const
 }
 
 
-void drumkv1::setOffsetRange ( uint32_t iOffsetStart, uint32_t iOffsetEnd )
+void drumkv1::setOffsetRange ( uint32_t iOffsetStart, uint32_t iOffsetEnd, bool bSync )
 {
 	m_pImpl->setOffsetRange(iOffsetStart, iOffsetEnd);
 	m_pImpl->sampleOffsetRangeSync();
 	m_pImpl->updateEnvTimes();
 
-	updateSample();
+	if (bSync) updateOffsetRange();
 }
 
 
@@ -2805,10 +2805,11 @@ int drumkv1_element::note (void) const
 void drumkv1_element::setSampleFile ( const char *pszSampleFile )
 {
 	if (m_pElem) {
-		m_pElem->gen1_sample.close();
 		if (pszSampleFile) {
 			m_pElem->gen1_sample.open(pszSampleFile,
 				drumkv1_freq(m_pElem->gen1.sample0));
+		} else {
+			m_pElem->gen1_sample.close();
 		}
 	}
 }
