@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -158,7 +158,7 @@ static void drumkv1_jack_session_event (
 // drumkv1_jack - impl.
 //
 
-drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
+drumkv1_jack::drumkv1_jack (const char *client_name) : drumkv1(2)
 {
 	m_client = nullptr;
 
@@ -186,7 +186,7 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 	drumkv1::programs()->enabled(true);
 	drumkv1::controls()->enabled(true);
 
-	open(DRUMKV1_TITLE);
+	open(client_name);
 	activate();
 }
 
@@ -210,7 +210,7 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 		return 0;
 
 	const uint16_t nchannels = drumkv1::channels();
-	float *ins[nchannels], *outs[nchannels];
+	float **ins = m_ins, **outs = m_outs;
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		ins[k]  = static_cast<float *> (
 			::jack_port_get_buffer(m_audio_ins[k], nframes));
@@ -244,8 +244,8 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 						outs[k] += nread;
 					}
 				}
+				ndelta = event.time;
 			}
-			ndelta = event.time;
 			drumkv1::process_midi(event.buffer, event.size);
 		}
 	}
@@ -288,7 +288,7 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 }
 
 
-void drumkv1_jack::open ( const char *client_id )
+void drumkv1_jack::open ( const char *client_name )
 {
 	// init param ports
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
@@ -298,7 +298,7 @@ void drumkv1_jack::open ( const char *client_id )
 	}
 
 	// open client
-	m_client = ::jack_client_open(client_id, JackNullOption, nullptr);
+	m_client = ::jack_client_open(client_name, JackNullOption, nullptr);
 	if (m_client == nullptr)
 		return;
 
@@ -341,7 +341,7 @@ void drumkv1_jack::open ( const char *client_id )
 	m_alsa_thread  = nullptr;
 	// open alsa sequencer client...
 	if (snd_seq_open(&m_alsa_seq, "hw", SND_SEQ_OPEN_INPUT, 0) >= 0) {
-		snd_seq_set_client_name(m_alsa_seq, client_id);
+		snd_seq_set_client_name(m_alsa_seq, client_name);
 	//	m_alsa_client = snd_seq_client_id(m_alsa_seq);
 		m_alsa_port = snd_seq_create_simple_port(m_alsa_seq, "in",
 			SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE,
@@ -608,16 +608,19 @@ void drumkv1_jack::sessionEvent ( void *pvSessionArg )
 #endif	// CONFIG_JACK_SESSION
 
 
-void drumkv1_jack::selectSample ( int key )
+void drumkv1_jack::updatePreset ( bool /*bDirty*/ )
 {
-	// something to do...
-	drumkv1::setCurrentElementEx(key);
-
-	drumkv1_sched::sync_notify(this, drumkv1_sched::Sample, 0);
+	// nothing to do here...
 }
 
 
-void drumkv1_jack::updatePreset ( bool /*bDirty*/ )
+void drumkv1_jack::updateParam ( drumkv1::ParamIndex /*index*/ )
+{
+	// nothing to do here...
+}
+
+
+void drumkv1_jack::updateParams (void)
 {
 	// nothing to do here...
 }
@@ -629,6 +632,21 @@ void drumkv1_jack::updateSample (void)
 }
 
 
+void drumkv1_jack::updateOffsetRange (void)
+{
+	// nothing to do here...
+}
+
+
+void drumkv1_jack::selectSample ( int key )
+{
+	// something to do...
+	drumkv1::setCurrentElementEx(key);
+
+	drumkv1_sched::sync_notify(this, drumkv1_sched::Sample, 0);
+}
+
+
 void drumkv1_jack::updateTuning (void)
 {
 	drumkv1::resetTuning();
@@ -696,7 +714,7 @@ static void drumkv1_sigterm_handler ( int /*signo*/ )
 // Constructor.
 drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 	: QObject(nullptr), m_pApp(nullptr), m_bGui(true),
-		m_pDrumk(nullptr), m_pWidget(nullptr)
+		m_sClientName(DRUMKV1_TITLE), m_pDrumk(nullptr), m_pWidget(nullptr)
 	  #ifdef CONFIG_NSM
 		, m_pNsmClient(nullptr)
 	  #endif
@@ -705,10 +723,8 @@ drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 	m_bGui = (::getenv("DISPLAY") != 0);
 #endif
 	for (int i = 1; i < argc; ++i) {
-		const QString& sArg = QString::fromLocal8Bit(argv[i]);
-		if (sArg[0] != '-')
-			m_presets.append(sArg);
-		else
+		const QString& sArg
+			= QString::fromLocal8Bit(argv[i]);
 		if (sArg == "-g" || sArg == "--no-gui")
 			m_bGui = false;
 	}
@@ -790,17 +806,42 @@ drumkv1_jack_application::~drumkv1_jack_application (void)
 bool drumkv1_jack_application::parse_args (void)
 {
 	QTextStream out(stderr);
-
 	const QStringList& args = m_pApp->arguments();
-	QStringListIterator iter(args);
-	while (iter.hasNext()) {
-		const QString& sArg = iter.next();
+	const int argc = args.count();
+
+	for (int i = 1; i < argc; ++i) {
+
+		QString sArg = args.at(i);
+
+		QString sVal;
+		const int iEqual = sArg.indexOf('=');
+		if (iEqual >= 0) {
+			sVal = sArg.right(sArg.length() - iEqual - 1);
+			sArg = sArg.left(iEqual);
+		}
+		else if (i < argc - 1) {
+			sVal = args.at(i + 1);
+			if (sVal.at(0) == '-')
+				sVal.clear();
+		}
+
+		if (sArg == "-n" || sArg == "--client-name") {
+			if (sVal.isNull()) {
+				out << QObject::tr("Option -n requires an argument (client-name).\n\n");
+				return false;
+			}
+			m_sClientName = sVal;
+			if (iEqual < 0)
+				++i;
+		}
+		else
 		if (sArg == "-h" || sArg == "--help") {
 			out << QObject::tr(
 				"Usage: %1 [options] [preset-file]\n\n"
 				DRUMKV1_TITLE " - " DRUMKV1_SUBTITLE "\n\n"
 				"Options:\n\n"
 				"  -g, --no-gui\n\tDisable the graphical user interface (GUI)\n\n"
+				"  -n, --client-name=[label]\n\tSet the JACK client name (default: drumkv1)\n\n"
 				"  -h, --help\n\tShow help about command line options\n\n"
 				"  -v, --version\n\tShow version information\n\n")
 				.arg(args.at(0));
@@ -815,6 +856,11 @@ bool drumkv1_jack_application::parse_args (void)
 				.arg(CONFIG_BUILD_VERSION);
 			return false;
 		}
+		else {
+			// If we don't have one by now,
+			// this will be the startup preset file...
+			m_presets.append(sArg);
+		}
 	}
 
 	return true;
@@ -836,7 +882,12 @@ bool drumkv1_jack_application::setup (void)
 		SIGNAL(shutdown_signal()),
 		SLOT(shutdown_slot()));
 
-	m_pDrumk = new drumkv1_jack();
+	const QByteArray aClientName
+		= m_sClientName.toLocal8Bit();
+	const char *client_name
+		= aClientName.constData();
+
+	m_pDrumk = new drumkv1_jack(client_name);
 
 	if (m_bGui) {
 		m_pWidget = new drumkv1widget_jack(m_pDrumk);
@@ -871,7 +922,7 @@ bool drumkv1_jack_application::setup (void)
 		QString caps(":switch:dirty:");
 		if (m_bGui)
 			caps += "optional-gui:";
-		m_pNsmClient->announce(DRUMKV1_TITLE, caps.toLatin1().constData());
+		m_pNsmClient->announce(DRUMKV1_TITLE, caps.toLocal8Bit().constData());
 		if (m_pWidget)
 			m_pWidget->setNsmClient(m_pNsmClient);
 	}
@@ -911,11 +962,11 @@ void drumkv1_jack_application::openSession (void)
 	m_pDrumk->deactivate();
 	m_pDrumk->close();
 
-	const QString& client_id = m_pNsmClient->client_id();
+	const QString& client_name = m_pNsmClient->client_name();
 	const QString& path_name = m_pNsmClient->path_name();
 	const QString& display_name = m_pNsmClient->display_name();
 
-	m_pDrumk->open(client_id.toUtf8().constData());
+	m_pDrumk->open(client_name.toUtf8().constData());
 	m_pDrumk->activate();
 
 	const QDir dir(path_name);
@@ -956,7 +1007,7 @@ void drumkv1_jack_application::saveSession (void)
 	qDebug("drumkv1_jack::saveSession()");
 #endif
 
-//	const QString& client_id = m_pNsmClient->client_id();
+//	const QString& client_name = m_pNsmClient->client_name();
 	const QString& path_name = m_pNsmClient->path_name();
 //	const QString& display_name = m_pNsmClient->display_name();
 //	const QFileInfo fi(path_name, display_name + '.' + DRUMKV1_TITLE);
