@@ -1,7 +1,7 @@
 // drumkv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -43,13 +43,13 @@ class drumkv1_jack : public drumkv1
 {
 public:
 
-	drumkv1_jack();
+	drumkv1_jack(const char *client_name);
 
 	~drumkv1_jack();
 
 	jack_client_t *client() const;
 
-	void open(const char *client_id);
+	void open(const char *client_name);
 	void close();
 
 	void activate();
@@ -73,9 +73,15 @@ public:
 protected:
 
 	void updatePreset(bool bDirty);
+	void updateParam(drumkv1::ParamIndex index);
+	void updateParams();
+
+	void updateSample();
+
+	void updateOffsetRange();
 
 	void selectSample(int key);
-	void updateSample();
+
 	void updateTuning();
 
 private:
@@ -182,6 +188,7 @@ private:
 	QCoreApplication *m_pApp;
 	bool m_bGui;
 
+	QString m_sClientName;
 	QStringList m_presets;
 
 	drumkv1_jack *m_pDrumk;
