@@ -1,7 +1,7 @@
 // drumkv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -56,6 +56,13 @@
 #define LV2_STATE__StateChanged LV2_STATE_PREFIX "StateChanged"
 #endif
 
+#ifndef LV2_ATOM__PortEvent
+#define LV2_ATOM__PortEvent LV2_ATOM_PREFIX "PortEvent"
+#endif
+#ifndef LV2_ATOM__portTuple
+#define LV2_ATOM__portTuple LV2_ATOM_PREFIX "portTuple"
+#endif
+
 #include <stdlib.h>
 #include <math.h>
 
@@ -127,9 +134,9 @@ private:
 typedef struct {
 	LV2_Atom atom;
 	union {
-		int key;
+		uint32_t    key;
 		const char *path;
-	} sample;
+	} data;
 } drumkv1_lv2_worker_message;
 
 
@@ -194,6 +201,12 @@ drumkv1_lv2::drumkv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
+			#ifdef CONFIG_LV2_PORT_EVENT
+				m_urids.atom_PortEvent = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__PortEvent);
+				m_urids.atom_portTuple = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__portTuple);
+			#endif
 				m_urids.time_Position = m_urid_map->map(
 					m_urid_map->handle, LV2_TIME__Position);
 				m_urids.time_beatsPerMinute = m_urid_map->map(
@@ -215,10 +228,6 @@ drumkv1_lv2::drumkv1_lv2 (
  					m_urid_map->handle, LV2_PATCH__Get);
 				m_urids.patch_Set = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__Set);
-				m_urids.patch_Put = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__Put);
-				m_urids.patch_body = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__body);
 				m_urids.patch_property = m_urid_map->map(
  					m_urid_map->handle, LV2_PATCH__property);
 				m_urids.patch_value = m_urid_map->map(
@@ -374,8 +383,8 @@ void drumkv1_lv2::run ( uint32_t nframes )
 							if (m_schedule) {
 								drumkv1_lv2_worker_message mesg;
 								mesg.atom.type = key;
-								mesg.atom.size = sizeof(mesg.sample);
-								mesg.sample.path
+								mesg.atom.size = sizeof(mesg.data.path);
+								mesg.data.path
 									= (const char *) LV2_ATOM_BODY_CONST(value);
 								// schedule loading new sample
 								m_schedule->schedule_work(
@@ -394,7 +403,7 @@ void drumkv1_lv2::run ( uint32_t nframes )
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
 								const uint32_t offset_end
 									= pSample->offsetEnd();
-								setOffsetRange(offset_start, offset_end);
+								setOffsetRange(offset_start, offset_end, true);
 							}
 						}
 						else
@@ -409,7 +418,7 @@ void drumkv1_lv2::run ( uint32_t nframes )
 									= pSample->offsetStart();
 								const uint32_t offset_end
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-								setOffsetRange(offset_start, offset_end);
+								setOffsetRange(offset_start, offset_end, true);
 							}
 						}
 						else
@@ -456,8 +465,14 @@ void drumkv1_lv2::run ( uint32_t nframes )
 				}
 				else
 				if (object->body.otype == m_urids.patch_Get) {
-					// put all property values (probably to UI)
-					patch_put(ndelta);
+					// get one or all property values (probably to UI)...
+					const LV2_Atom_URID *prop = nullptr;
+					lv2_atom_object_get(object,
+						m_urids.patch_property, (const LV2_Atom *) &prop, 0);
+					if (prop && prop->atom.type == m_forge.URID)
+						patch_get(prop->body);
+					else
+						patch_get(0); // all
 				}
 			#endif	// CONFIG_LV2_PATCH
 			}
@@ -505,12 +520,12 @@ void drumkv1_lv2::qapp_instantiate (void)
 {
 	if (qApp == nullptr && g_qapp_instance == nullptr) {
 		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, nullptr };
+		static const char *s_argv[] = { DRUMKV1_TITLE, nullptr };
+		::setenv("QT_NO_GLIB", "1", 1); // Avoid glib event-loop...
 		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
 	}
 
-	if (g_qapp_instance)
-		g_qapp_refcount++;
+	if (g_qapp_instance) ++g_qapp_refcount;
 }
 
 
@@ -703,20 +718,64 @@ void drumkv1_lv2::updatePreset ( bool /*bDirty*/ )
 		drumkv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.state_StateChanged;
 		mesg.atom.size = 0; // nothing else matters.
-		mesg.sample.path = nullptr;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
 }
 
 
+void drumkv1_lv2::updateParam ( drumkv1::ParamIndex index )
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		drumkv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = sizeof(mesg.data.key);
+		mesg.data.key  = uint32_t(index);
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#else
+	(void) index; // STFU dang compiler!
+#endif
+}
+
+
+void drumkv1_lv2::updateParams (void)
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		drumkv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+		m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#endif
+}
+
+
 void drumkv1_lv2::updateSample (void)
 {
 	if (m_schedule) {
 		drumkv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.gen1_update;
-		mesg.atom.size = sizeof(mesg.sample);
-		mesg.sample.path = drumkv1::sampleFile();
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void drumkv1_lv2::updateOffsetRange (void)
+{
+	if (m_schedule) {
+		drumkv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.p102_offset_start;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+		mesg.atom.type = m_urids.p103_offset_end;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
@@ -728,8 +787,8 @@ void drumkv1_lv2::selectSample ( int key )
 	if (m_schedule) {
 		drumkv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.gen1_select;
-		mesg.atom.size = sizeof(mesg.sample);
-		mesg.sample.key = key;
+		mesg.atom.size = sizeof(mesg.data.key);
+		mesg.data.key = key;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
@@ -756,33 +815,22 @@ bool drumkv1_lv2::worker_work ( const void *data, uint32_t size )
 	const drumkv1_lv2_worker_message *mesg
 		= (const drumkv1_lv2_worker_message *) data;
 
-	if (mesg->atom.type == m_urids.state_StateChanged)
-		return true;
+	if (mesg->atom.type == m_urids.gen1_select)
+		drumkv1::setCurrentElementEx(mesg->data.key);
 	else
-	if (mesg->atom.type == m_urids.gen1_update)
-		return true;
-	else
-	if (mesg->atom.type == m_urids.gen1_select) {
-		drumkv1::setCurrentElementEx(mesg->sample.key);
-		return true;
-	}
-	else
-	if (mesg->atom.type == m_urids.gen1_sample) {
+	if (mesg->atom.type == m_urids.p101_sample_file) {
 		const int key = drumkv1::currentElement();
 		if (drumkv1::element(key) == nullptr) {
 			drumkv1::addElement(key);
 			drumkv1::setCurrentElementEx(key);
 		}
-		drumkv1::setSampleFile(mesg->sample.path);
-		return true;
+		drumkv1::setSampleFile(mesg->data.path);
 	}
 	else
-	if (mesg->atom.type == m_urids.tun1_update) {
+	if (mesg->atom.type == m_urids.tun1_update)
 		drumkv1::resetTuning();
-		return true;
-	}
 
-	return false;
+	return true;
 }
 
 
@@ -793,6 +841,19 @@ bool drumkv1_lv2::worker_response ( const void *data, uint32_t size )
 
 	const drumkv1_lv2_worker_message *mesg
 		= (const drumkv1_lv2_worker_message *) data;
+
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (mesg->atom.type == m_urids.atom_PortEvent) {
+		if (mesg->atom.size > 0)
+			return port_event(drumkv1::ParamIndex(mesg->data.key));
+		else
+			return port_events(drumkv1::NUM_PARAMS);
+	}
+	else
+	if (mesg->atom.type == m_urids.gen1_select)
+		port_events(drumkv1::NUM_ELEMENT_PARAMS);
+	else
+#endif
 	if (mesg->atom.type == m_urids.state_StateChanged)
 		return state_changed();
 
@@ -800,7 +861,7 @@ bool drumkv1_lv2::worker_response ( const void *data, uint32_t size )
 	drumkv1_sched::sync_notify(this, drumkv1_sched::Sample, 0);
 
 #ifdef CONFIG_LV2_PATCH
-	return patch_put(m_ndelta, mesg->atom.type);
+	return patch_get(mesg->atom.type);
 #else
 	return true;
 #endif
@@ -821,78 +882,142 @@ bool drumkv1_lv2::state_changed (void)
 
 #ifdef CONFIG_LV2_PATCH
 
-bool drumkv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
+bool drumkv1_lv2::patch_set ( LV2_URID key )
 {
 	static char s_szNull[1] = {'\0'};
 
-	if (type == m_urids.patch_Put)
-		type = 0;
-
 	drumkv1_sample *pSample = drumkv1::sample();
 	if (pSample == nullptr)
 		return false;
 
-	lv2_atom_forge_frame_time(&m_forge, ndelta);
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
 
 	LV2_Atom_Forge_Frame patch_frame;
-	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Put);
-	lv2_atom_forge_key(&m_forge, m_urids.patch_body);
+	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Set);
 
-	LV2_Atom_Forge_Frame body_frame;
-	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_property);
+	lv2_atom_forge_urid(&m_forge, key);
 
-	if (type == 0 || type == m_urids.p101_sample_file) {
+	lv2_atom_forge_key(&m_forge, m_urids.patch_value);
+
+	if (key == m_urids.p101_sample_file) {
 		const char *pszSampleFile = pSample->filename();
 		if (pszSampleFile == nullptr)
 			pszSampleFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p101_sample_file);
 		lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
 	}
-	if (type == 0 || type == m_urids.p102_offset_start) {
-		lv2_atom_forge_key(&m_forge, m_urids.p102_offset_start);
+	else
+	if (key == m_urids.p102_offset_start)
 		lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetStart() : 0));
-	}
-	if (type == 0 || type == m_urids.p103_offset_end) {
-		lv2_atom_forge_key(&m_forge, m_urids.p103_offset_end);
+	else
+	if (key == m_urids.p103_offset_end)
 		lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetEnd() : 0));
-	}
-
-	if (type == 0 || type == m_urids.p201_tuning_enabled) {
-		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+	else
+	if (key == m_urids.p201_tuning_enabled)
 		lv2_atom_forge_bool(&m_forge, drumkv1::isTuningEnabled());
-	}
-	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
-		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+	else
+	if (key == m_urids.p202_tuning_refPitch)
 		lv2_atom_forge_float(&m_forge, drumkv1::tuningRefPitch());
-	}
-	if (type == 0 || type == m_urids.p203_tuning_refNote) {
-		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+	else
+	if (key == m_urids.p203_tuning_refNote)
 		lv2_atom_forge_int(&m_forge, drumkv1::tuningRefNote());
-	}
-	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+	else
+	if (key == m_urids.p204_tuning_scaleFile) {
 		const char *pszScaleFile = drumkv1::tuningScaleFile();
 		if (pszScaleFile == nullptr)
 			pszScaleFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
 		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
 	}
-	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+	else
+	if (key == m_urids.p205_tuning_keyMapFile) {
 		const char *pszKeyMapFile = drumkv1::tuningKeyMapFile();
 		if (pszKeyMapFile == nullptr)
 			pszKeyMapFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
 		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
 	}
 
-	lv2_atom_forge_pop(&m_forge, &body_frame);
 	lv2_atom_forge_pop(&m_forge, &patch_frame);
 
 	return true;
 }
 
+bool drumkv1_lv2::patch_get ( LV2_URID key )
+{
+	if (key == 0 || key == m_urids.gen1_update || key == m_urids.gen1_select) {
+		patch_set(m_urids.p101_sample_file);
+		patch_set(m_urids.p102_offset_start);
+		patch_set(m_urids.p103_offset_end);
+		if (key) return true;
+	}
+
+	if (key == 0 || key == m_urids.tun1_update) {
+		patch_set(m_urids.p201_tuning_enabled);
+		patch_set(m_urids.p202_tuning_refPitch);
+		patch_set(m_urids.p203_tuning_refNote);
+		patch_set(m_urids.p204_tuning_scaleFile);
+		patch_set(m_urids.p205_tuning_keyMapFile);
+		if (key) return true;
+	}
+
+	if (key) patch_set(key);
+
+	return true;
+}
+
 #endif	// CONFIG_LV2_PATCH
 
 
+#ifdef CONFIG_LV2_PORT_EVENT
+
+bool drumkv1_lv2::port_event ( drumkv1::ParamIndex index )
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+	lv2_atom_forge_float(&m_forge, drumkv1::paramValue(index));
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+
+bool drumkv1_lv2::port_events ( uint32_t nparams )
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	for (uint32_t i = 0; i < nparams; ++i) {
+		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		if (index == drumkv1::GEN1_SAMPLE)
+			continue;
+		lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+		lv2_atom_forge_float(&m_forge, drumkv1::paramValue(index));
+	}
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+#endif	// CONFIG_LV2_PORT_EVENT
+
+
 //-------------------------------------------------------------------------
 // drumkv1_lv2 - LV2 desc.
 //
