@@ -1,7 +1,7 @@
 // drumkv1_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -54,9 +54,11 @@ bool drumkv1_sample::open ( const char *filename, float freq0 )
 	if (filename == nullptr)
 		return false;
 
+	char *filename2 = ::strdup(filename);
+
 	close();
 
-	m_filename = ::strdup(filename);
+	m_filename = filename2;
 
 	SF_INFO info;
 	::memset(&info, 0, sizeof(info));
@@ -141,7 +143,7 @@ void drumkv1_sample::close (void)
 	m_rate0     = 0.0f;
 	m_nchannels = 0;
 
-	setOffsetRange(0, 0);
+//	setOffsetRange(0, 0);
 
 	if (m_filename) {
 		::free(m_filename);
