@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -52,8 +52,8 @@
 //
 
 // Constructor.
-drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
-	: QWidget(pParent, wflags), p_ui(new Ui::drumkv1widget), m_ui(*p_ui)
+drumkv1widget::drumkv1widget ( QWidget *pParent )
+	: QWidget(pParent), p_ui(new Ui::drumkv1widget), m_ui(*p_ui)
 {
 	Q_INIT_RESOURCE(drumkv1);
 
@@ -94,7 +94,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Offset/Loop range font.
 	const QFont& font = m_ui.Gen1ReverseKnob->font();
-	m_ui.Gen1OffsetRangeLabel->setFont(font);
+	const QFont font2(font.family(), font.pointSize() - 1);
+	m_ui.Gen1OffsetRangeLabel->setFont(font2);
 	m_ui.Gen1OffsetStartSpinBox->setFont(font);
 	m_ui.Gen1OffsetEndSpinBox->setFont(font);
 
@@ -453,6 +454,10 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(drumkv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(drumkv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
 
+    // added by scotty to try to support linnstruments need for bigger bend range
+    m_ui.Def1PitchbendKnob->setMinimum(0.0f);
+    m_ui.Def1PitchbendKnob->setMaximum(+4.0f);
+
 
 	// Element selectors...
 	QObject::connect(m_ui.Elements,
@@ -528,6 +533,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(toggled(bool)),
 		SLOT(swapParams(bool)));
 
+	// Randomize params...
+	QObject::connect(m_ui.PanicButton,
+		SIGNAL(clicked()),
+		SLOT(panic()));
+	
 	// Direct stacked-page signal/slot
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
@@ -847,9 +857,9 @@ void drumkv1widget::randomParams (void)
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		// Filter out some non-randomizable parameters!...
 		if (index == drumkv1::GEN1_SAMPLE   ||
-		    index == drumkv1::GEN1_OFFSET   ||
-		    index == drumkv1::GEN1_OFFSET_1 ||
-		    index == drumkv1::GEN1_OFFSET_2 ||
+			index == drumkv1::GEN1_OFFSET   ||
+			index == drumkv1::GEN1_OFFSET_1 ||
+			index == drumkv1::GEN1_OFFSET_2 ||
 			index == drumkv1::GEN1_GROUP    ||
 			index == drumkv1::GEN1_COARSE   ||
 			index == drumkv1::GEN1_FINE     ||
@@ -947,6 +957,15 @@ void drumkv1widget::swapParams ( bool bOn )
 }
 
  
+// Panic: all-notes/sound-off (reset).
+void drumkv1widget::panic (void)
+{
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->reset();
+}
+
+
 // Reset swap params A/B group.
 void drumkv1widget::resetSwapParams (void)
 {
@@ -1545,7 +1564,7 @@ void drumkv1widget::updateOffset ( drumkv1_sample *pSample, bool bDirty )
 					.arg(m_ui.Gen1Sample->textFromValue(iOffsetEnd)));
 			}
 			if (!sMessage.isEmpty())
-				m_ui.StatusBar->showMessage(sMessage, 3000);
+				m_ui.StatusBar->showMessage(sMessage, 5000);
 			updateDirtyPreset(true);
 		}
 	} else {
