@@ -1,7 +1,7 @@
 // drumkv1widget_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -152,7 +152,7 @@ drumkv1widget_param::drumkv1widget_param ( QWidget *pParent ) : QWidget(pParent)
 	QWidget::setMaximumSize(QSize(52, 72));
 
 	QGridLayout *pGridLayout = new QGridLayout();
-	pGridLayout->setMargin(0);
+	pGridLayout->setContentsMargins(0, 0, 0, 0);
 	pGridLayout->setSpacing(0);
 	QWidget::setLayout(pGridLayout);
 }
@@ -730,7 +730,7 @@ void drumkv1widget_combo::comboBoxValueChanged ( int iComboValue )
 void drumkv1widget_combo::wheelEvent ( QWheelEvent *pWheelEvent )
 {
 	const int delta
-		= (pWheelEvent->delta() / 120);
+		= (pWheelEvent->angleDelta().y() / 120);
 	if (delta) {
 		float fValue = value() + float(delta);
 		if (fValue < minimum())
@@ -761,7 +761,11 @@ drumkv1widget_radio::drumkv1widget_radio ( QWidget *pParent )
 #endif
 
 	QObject::connect(&m_group,
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+		SIGNAL(idClicked(int)),
+	#else
 		SIGNAL(buttonClicked(int)),
+	#endif
 		SLOT(radioGroupValueChanged(int)));
 }
 
@@ -961,6 +965,7 @@ drumkv1widget_group::drumkv1widget_group ( QWidget *pParent )
 
 	m_pParam = new drumkv1widget_param(this);
 	m_pParam->setToolTip(QGroupBox::toolTip());
+	m_pParam->setValue(0.5f); // HACK: half-way on.
 
 	QObject::connect(m_pParam,
 		 SIGNAL(valueChanged(float)),
