@@ -1,7 +1,7 @@
 // drumkv1widget_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -49,9 +49,8 @@
 // drumkv1widget_sample -- Custom widget
 
 // Constructor.
-drumkv1widget_sample::drumkv1widget_sample (
-	QWidget *pParent, Qt::WindowFlags wflags )
-	: QFrame(pParent, wflags), m_pSample(nullptr), m_iChannels(0), m_ppPolyg(nullptr)
+drumkv1widget_sample::drumkv1widget_sample ( QWidget *pParent )
+	: QFrame(pParent), m_pSample(nullptr), m_iChannels(0), m_ppPolyg(nullptr)
 {
 	QFrame::setMouseTracking(true);
 	QFrame::setFocusPolicy(Qt::ClickFocus);
@@ -626,7 +625,7 @@ void drumkv1widget_sample::openSample ( const QString& sName )
 	const QString& sTitle  = tr("Open Sample [%1]").arg(sName);
 	const QString& sFilter = s_filters.join(";;");
 	QWidget *pParentWidget = nullptr;
-	QFileDialog::Options options = 0;
+	QFileDialog::Options options;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
 		pParentWidget = QWidget::window();
@@ -693,9 +692,16 @@ void drumkv1widget_sample::updateToolTip (void)
 	const char *pszSampleFile = (m_pSample ? m_pSample->filename() : 0);
 	if (pszSampleFile) {
 		if (!sToolTip.isEmpty()) sToolTip += '\n';
-		sToolTip += tr("%1\n%2 frames, %3 channels, %4 Hz")
+		QString suffix;
+		drumkv1widget_spinbox::Format format = drumkv1widget_spinbox::Frames;
+		drumkv1_config *pConfig = drumkv1_config::getInstance();
+		if (pConfig)
+			format = drumkv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+		if (format == drumkv1widget_spinbox::Frames)
+			suffix = tr(" frames");
+		sToolTip += tr("%1\n%2%3, %4 channels, %5 Hz")
 			.arg(QFileInfo(pszSampleFile).completeBaseName())
-			.arg(m_pSample->length())
+			.arg(m_pSample->length()).arg(suffix)
 			.arg(m_pSample->channels())
 			.arg(m_pSample->rate());
 	}
